// Name
// Student ID

import controlP5.*;

// The user interface: a Textfield to hold
// the query word, and a Textarea to hold all
// the rhyming words.
ControlP5 ui;
Textfield field;
Textarea area;

// A simple class that collects two things:
// a word, and its rhyming pattern (also 
// represnted as a string).
class Word
{
  String word;
  String rhyme;

  Word( String w, String r )
  {
    word = w;
    rhyme = r;
  }
}

// Determine the stress level of a phoneme.  The
// stress level is an integer -- either 0 (no stress),
// 1 (primary stress), or 2 (secondary stress).  If
// the phoneme isn't a vowel sound then there's no 
// implied stress and the function returns -1.
int getStress( String phoneme )
{
  // See if the phoneme contains any of the digits
  // 0, 1, or 2.
  for ( int idx = 0; idx < phoneme.length(); ++idx ) {
    char c = phoneme.charAt( idx );
    if ( c == '0' ) {
      return 0;
    } else if ( c == '1' ) {
      return 1;
    } else if ( c == '2' ) {
      return 2;
    }
  }
  return -1;
}

// Get a magic string that encodes a word's rhyming
// structure.  A word's rhyme is the complete set of
// phonemes running from the final stressed vowel
// to the end of the word, strung together into one
// long string.  If there are no stressed
// vowels in the word, return the whole pronunciation
// as its rhyme.
//
// Examples:
// word: "backpack"
// pronunciation: "B AE1 K P AE2 K"
// rhyming phonemes: "AE2 K"
// stored rhyming pattern: "KAE2"
//
// word: "timber"
// pronunciation: "T IH1 M B ER0"
// rhyming phonemes: "IH1 M B ER0"
// stored rhyming pattern: "ER0BMIH1"
//  
String getRhyme( String[] pron )
{
  String ret = "";
  // Walk backwards over the array of phonemes until
  // you spot a stressed one.
  for ( int idx = pron.length - 1; idx >= 1; --idx ) {
    ret = ret + pron[idx];
    if ( getStress( pron[idx] ) > 0 ) {
      break;
    }
  }

  return ret;
}

// A sequence of word / pronunciation pairs.
Word[] words;

// Given a word, search the list of word / pronunciation
// pairs to find all the other words that rhyme with it.
// Do this in two passes over the array (i.e., for loops):
// first, find the rhyme pattern for the given word by
// looking it up in the word list.  Then find all other
// words that have the same rhyme pattern.
String[] getRhymes( String word )
{
  String[] ret = {};

  // TODO: fill in this function.  Find the word in
  // the array in order to figure out its associated
  // rhyming pattern.  Then find all the words in the
  // array that have the same rhyming pattern.  When
  // you find them, append them to the array "ret".

  return ret;
}

void setup()
{
  size( 300, 600 );

  words = new Word[0];

  // TODO -- load the file pronunciation.txt.  Now
  // walk over it line by line.  If the line starts with 
  // ";;;", skip it.  Otherwise, use splitTokens() to turn
  // the line into an array of the word followed by its
  // phonemes.  Use the helper function above to get the
  // rhyme pattern for the word, and store the combination
  // of the word and its rhyme in a new Word object.  
  // Then append that object to the array of words.
  // Use something like this:
  // Word nw = new Word( ..... );
  // words = (Word[])append( words, nw );
  
  PFont fnt = createFont( "Helvetica", 20 );
  ui = new ControlP5( this );
  field = ui.addTextfield( "Word" )
    .setPosition( 5, 5 )
    .setSize( 290, 25 )
    .setAutoClear( false )
    .setFont( fnt );

  area = ui.addTextarea( "Rhymes" )
    .setPosition( 5, 50 )
    .setSize( 290, height - 55 )
    .setFont( fnt );
}

void draw()
{
  background( 0 );
}

void controlEvent( ControlEvent ce )
{
  // This event will fire when the user presses
  // return in the text field.  Convert it to 
  // upper case to be compatible with the words
  // stored in the pronunciation dictionary.
  String wrd = field.getText().toUpperCase();
  String[] rhms = getRhymes( wrd );
  if ( rhms.length == 0 ) {
    area.setText( "Word not found" );
  } else {
    area.setText( join( rhms, "\n" ) );
  }
}
